@@ -13,6 +13,7 @@
 
 #include "dbm_hyperparams.h"
 #include "dbm_mempool.h"
+#include "dbm_mpi.h"
 
 /*******************************************************************************
  * \brief Returns the larger of two given integer (missing from the C standard)
@@ -252,8 +253,8 @@ static void fill_send_buffers(
  * \author Ole Schuett
  ******************************************************************************/
 static int compare_pack_blocks_by_sum_index(const void *a, const void *b) {
-  const dbm_pack_block_t *blk_a = (dbm_pack_block_t *)a;
-  const dbm_pack_block_t *blk_b = (dbm_pack_block_t *)b;
+  const dbm_pack_block_t *blk_a = (const dbm_pack_block_t *)a;
+  const dbm_pack_block_t *blk_b = (const dbm_pack_block_t *)b;
   return blk_a->sum_index - blk_b->sum_index;
 }
 
@@ -353,14 +354,18 @@ static dbm_packed_matrix_t pack_matrix(const bool trans_matrix,
                     dist_ticks, nticks, nsend_packs, plans_per_pack,
                     nblks_send_per_pack, ndata_send_per_pack);
 
-  // Can not parallelize over packs because there might be too few of them.
-  for (int ipack = 0; ipack < nsend_packs; ipack++) {
-    // Allocate send buffers.
-    dbm_pack_block_t *blks_send =
-        malloc(nblks_send_per_pack[ipack] * sizeof(dbm_pack_block_t));
-    double *data_send =
-        dbm_mempool_host_malloc(ndata_send_per_pack[ipack] * sizeof(double));
+  // Allocate send buffers for maximum number of blocks/data over all packs.
+  int nblks_send_max = 0, ndata_send_max = 0;
+  for (int ipack = 0; ipack < nsend_packs; ++ipack) {
+    nblks_send_max = imax(nblks_send_max, nblks_send_per_pack[ipack]);
+    ndata_send_max = imax(ndata_send_max, ndata_send_per_pack[ipack]);
+  }
+  dbm_pack_block_t *blks_send =
+      dbm_mpi_alloc_mem(nblks_send_max * sizeof(dbm_pack_block_t));
+  double *data_send = dbm_mempool_host_malloc(ndata_send_max * sizeof(double));
 
+  // Cannot parallelize over packs (there might be too few of them).
+  for (int ipack = 0; ipack < nsend_packs; ipack++) {
     // Fill send buffers according to plans.
     const int nranks = dist->nranks;
     int blks_send_count[nranks], data_send_count[nranks];
@@ -379,7 +384,7 @@ static dbm_packed_matrix_t pack_matrix(const bool trans_matrix,
 
     // 2nd communication: Exchange blocks.
     dbm_pack_block_t *blks_recv =
-        malloc(nblocks_recv * sizeof(dbm_pack_block_t));
+        dbm_mpi_alloc_mem(nblocks_recv * sizeof(dbm_pack_block_t));
     int blks_send_count_byte[nranks], blks_send_displ_byte[nranks];
     int blks_recv_count_byte[nranks], blks_recv_displ_byte[nranks];
     for (int i = 0; i < nranks; i++) { // TODO: this is ugly!
@@ -391,7 +396,6 @@ static dbm_packed_matrix_t pack_matrix(const bool trans_matrix,
     dbm_mpi_alltoallv_byte(
         blks_send, blks_send_count_byte, blks_send_displ_byte, blks_recv,
         blks_recv_count_byte, blks_recv_displ_byte, dist->comm);
-    free(blks_send);
 
     // 3rd communication: Exchange data counts.
     // TODO: could be computed from blks_recv.
@@ -405,7 +409,6 @@ static dbm_packed_matrix_t pack_matrix(const bool trans_matrix,
     dbm_mpi_alltoallv_double(data_send, data_send_count, data_send_displ,
                              data_recv, data_recv_count, data_recv_displ,
                              dist->comm);
-    dbm_mempool_free(data_send);
 
     // Post-process received blocks and assemble them into a pack.
     postprocess_received_blocks(nranks, dist_indices->nshards, nblocks_recv,
@@ -417,6 +420,10 @@ static dbm_packed_matrix_t pack_matrix(const bool trans_matrix,
     packed.send_packs[ipack].data = data_recv;
   }
 
+  // Deallocate send buffers.
+  dbm_mpi_free_mem(blks_send);
+  dbm_mempool_free(data_send);
+
   // Allocate pack_recv.
   int max_nblocks = 0, max_data_size = 0;
   for (int ipack = 0; ipack < packed.nsend_packs; ipack++) {
@@ -428,7 +435,7 @@ static dbm_packed_matrix_t pack_matrix(const bool trans_matrix,
   packed.max_nblocks = max_nblocks;
   packed.max_data_size = max_data_size;
   packed.recv_pack.blocks =
-      malloc(packed.max_nblocks * sizeof(dbm_pack_block_t));
+      dbm_mpi_alloc_mem(packed.max_nblocks * sizeof(dbm_pack_block_t));
   packed.recv_pack.data =
       dbm_mempool_host_malloc(packed.max_data_size * sizeof(double));
 
@@ -451,7 +458,7 @@ static dbm_pack_t *sendrecv_pack(const int itick, const int nticks,
   const int send_ipack = send_itick / nranks;
   assert(send_itick % nranks == my_rank);
 
-  // Compute recevice rank and pack.
+  // Compute receive rank and pack.
   const int recv_rank = itick % nranks;
   const int recv_ipack = itick / nranks;
 
@@ -497,10 +504,10 @@ static dbm_pack_t *sendrecv_pack(const int itick, const int nticks,
  * \author Ole Schuett
  ******************************************************************************/
 static void free_packed_matrix(dbm_packed_matrix_t *packed) {
-  free(packed->recv_pack.blocks);
+  dbm_mpi_free_mem(packed->recv_pack.blocks);
   dbm_mempool_free(packed->recv_pack.data);
   for (int ipack = 0; ipack < packed->nsend_packs; ipack++) {
-    free(packed->send_packs[ipack].blocks);
+    dbm_mpi_free_mem(packed->send_packs[ipack].blocks);
     dbm_mempool_free(packed->send_packs[ipack].data);
   }
   free(packed->send_packs);
