@@ -34,19 +34,18 @@ void dbm_multiply_gpu_start(const int max_batch_size, const int nshards,
 
   // Allocate device storage for batches.
   const size_t size = nshards * max_batch_size * sizeof(dbm_task_t);
-  ctx->batches_dev = (dbm_task_t *)dbm_mempool_device_malloc(size);
+  ctx->batches_dev = dbm_mempool_device_malloc(size);
 
   // Allocate and upload shards of result matrix C.
-  ctx->shards_c_dev =
-      (dbm_shard_gpu_t *)malloc(nshards * sizeof(dbm_shard_gpu_t));
+  ctx->shards_c_dev = malloc(nshards * sizeof(dbm_shard_gpu_t));
   for (int i = 0; i < nshards; i++) {
     const dbm_shard_t *shard_c_host = &ctx->shards_c_host[i];
     dbm_shard_gpu_t *shard_c_dev = &ctx->shards_c_dev[i];
     offloadStreamCreate(&shard_c_dev->stream);
     shard_c_dev->data_size = shard_c_host->data_size;
     shard_c_dev->data_allocated = shard_c_host->data_allocated;
-    shard_c_dev->data = (double *)dbm_mempool_device_malloc(
-        shard_c_dev->data_allocated * sizeof(double));
+    shard_c_dev->data =
+        dbm_mempool_device_malloc(shard_c_dev->data_allocated * sizeof(double));
     offloadMemcpyAsyncHtoD(shard_c_dev->data, shard_c_host->data,
                            shard_c_dev->data_size * sizeof(double),
                            shard_c_dev->stream);
@@ -63,7 +62,7 @@ static void upload_pack(const dbm_pack_t *pack_host, dbm_pack_t *pack_dev,
   const size_t size = pack_host->data_size * sizeof(double);
   if (pack_dev->data_size < pack_host->data_size) {
     dbm_mempool_free(pack_dev->data);
-    pack_dev->data = (double *)dbm_mempool_device_malloc(size);
+    pack_dev->data = dbm_mempool_device_malloc(size);
   }
   offloadMemcpyAsyncHtoD(pack_dev->data, pack_host->data, size, stream);
 }
@@ -102,6 +101,7 @@ void dbm_multiply_gpu_upload_packs(const dbm_pack_t *pack_a,
  * \author Ole Schuett
  ******************************************************************************/
 void dbm_multiply_gpu_process_batch(const int ntasks, const dbm_task_t *batch,
+                                    const int mnk_range[3][2],
                                     const double alpha, const int kshard,
                                     dbm_multiply_gpu_context_t *ctx) {
 
@@ -123,13 +123,13 @@ void dbm_multiply_gpu_process_batch(const int ntasks, const dbm_task_t *batch,
   offloadEventCreate(&batch_uploaded);
   offloadEventRecord(batch_uploaded, shard_c_dev->stream);
 
-  // Reallocate shard_c_dev->data if nessecary.
+  // Reallocate shard_c_dev->data if necessary.
   if (shard_c_host->data_promised > shard_c_dev->data_allocated) {
     double *old_data_dev = shard_c_dev->data;
     shard_c_dev->data_allocated =
         ALLOCATION_FACTOR * shard_c_host->data_promised;
-    shard_c_dev->data = (double *)dbm_mempool_device_malloc(
-        shard_c_dev->data_allocated * sizeof(double));
+    shard_c_dev->data =
+        dbm_mempool_device_malloc(shard_c_dev->data_allocated * sizeof(double));
     offloadMemcpyAsyncDtoD(shard_c_dev->data, old_data_dev,
                            shard_c_dev->data_size * sizeof(double),
                            shard_c_dev->stream);
@@ -138,7 +138,7 @@ void dbm_multiply_gpu_process_batch(const int ntasks, const dbm_task_t *batch,
     dbm_mempool_free(old_data_dev);
   }
 
-  // Zero new blocks if nessecary.
+  // Zero new blocks if necessary.
   if (shard_c_host->data_promised > shard_c_dev->data_size) {
     const int tail = shard_c_host->data_promised - shard_c_dev->data_size;
     offloadMemsetAsync(&shard_c_dev->data[shard_c_dev->data_size], 0,
@@ -147,9 +147,9 @@ void dbm_multiply_gpu_process_batch(const int ntasks, const dbm_task_t *batch,
   }
 
   // Launch kernel.
-  dbm_multiply_gpu_launch_kernel(shard_c_dev->stream, alpha, ntasks, batch_dev,
-                                 ctx->pack_a_dev.data, ctx->pack_b_dev.data,
-                                 shard_c_dev->data);
+  dbm_multiply_gpu_launch_kernel(shard_c_dev->stream, mnk_range, alpha, ntasks,
+                                 batch_dev, ctx->pack_a_dev.data,
+                                 ctx->pack_b_dev.data, shard_c_dev->data);
   OFFLOAD_CHECK(offloadGetLastError());
 
   // Wait for batch to be uploaded before refilling it.
@@ -167,7 +167,7 @@ void dbm_multiply_gpu_download_results(dbm_multiply_gpu_context_t *ctx) {
 
 #pragma omp parallel for schedule(dynamic)
   for (int i = 0; i < ctx->nshards; i++) {
-    // Grow host buffer if nessecary.
+    // Grow host buffer if necessary.
     dbm_shard_t *shard_c_host = &ctx->shards_c_host[i];
     dbm_shard_allocate_promised_blocks(shard_c_host);
 
