@@ -102,7 +102,7 @@ void grid_sphere_cache_lookup(const double radius, const double dh[3][3],
 
   // Find or create cache entry for given grid.
   const double dr0 = dh[0][0], dr1 = dh[1][1], dr2 = dh[2][2];
-  grid_sphere_cache_entry *entry;
+  grid_sphere_cache_entry *entry = 0;
   bool found = false;
 
   // Fast path: check prev match.
