@@ -12,13 +12,13 @@
 #include "offload_buffer.h"
 #include "offload_library.h"
 #include "offload_runtime.h"
+
 /*******************************************************************************
- * \brief Allocates a buffer of given length, ie. number of elements.
+ * \brief Allocates a buffer of given length, ie., number of elements.
  * \author Ole Schuett
  ******************************************************************************/
 void offload_create_buffer(const int length, offload_buffer **buffer) {
-
-  const size_t requested_size = length * sizeof(double);
+  const size_t requested_size = sizeof(double) * length;
 
   if (*buffer != NULL) {
     if ((*buffer)->size >= requested_size) {
@@ -32,29 +32,17 @@ void offload_create_buffer(const int length, offload_buffer **buffer) {
   (*buffer)->size = requested_size;
   (*buffer)->host_buffer = NULL;
   (*buffer)->device_buffer = NULL;
+#if defined(__OFFLOAD)
   offload_activate_chosen_device();
-
-#if defined(__OFFLOAD_CUDA)
-  // With size 0 cudaMallocHost doesn't null the pointer and cudaFreeHost fails.
-  OFFLOAD_CHECK(
-      cudaMallocHost((void **)&(*buffer)->host_buffer, requested_size));
-  OFFLOAD_CHECK(cudaMalloc((void **)&(*buffer)->device_buffer, requested_size));
-#elif defined(__OFFLOAD_HIP)
-  // With size 0 cudaMallocHost doesn't null the pointer and cudaFreeHost fails.
-#ifndef __OFFLOAD_UNIFIED_MEMORY
-  OFFLOAD_CHECK(hipHostMalloc((void **)&(*buffer)->host_buffer, requested_size,
-                              hipHostMallocDefault));
-  OFFLOAD_CHECK(hipMalloc((void **)&(*buffer)->device_buffer, requested_size));
-#else
-  OFFLOAD_CHECK(hipMalloc((void **)&(*buffer)->host_buffer, requested_size));
-  (*buffer)->device_buffer = (*buffer)->host_buffer;
-#endif
-
+  offloadMallocHost((void **)&(*buffer)->host_buffer, requested_size);
+  offloadMalloc((void **)&(*buffer)->device_buffer, requested_size);
 #else
   (*buffer)->host_buffer = malloc(requested_size);
   (*buffer)->device_buffer = NULL;
 #endif
-  return;
+  if (NULL == (*buffer)->host_buffer) { /* unified memory */
+    (*buffer)->host_buffer = (*buffer)->device_buffer;
+  }
 }
 
 /*******************************************************************************
@@ -62,23 +50,15 @@ void offload_create_buffer(const int length, offload_buffer **buffer) {
  * \author Ole Schuett
  ******************************************************************************/
 void offload_free_buffer(offload_buffer *buffer) {
-
-  if (buffer == NULL)
+  if (NULL == buffer)
     return;
-
-#ifdef __OFFLOAD_CUDA
-  OFFLOAD_CHECK(cudaFreeHost(buffer->host_buffer));
-  OFFLOAD_CHECK(cudaFree(buffer->device_buffer));
-#elif defined(__OFFLOAD_HIP)
-  OFFLOAD_CHECK(hipFree(buffer->device_buffer));
-#ifndef __OFFLOAD_UNIFIED_MEMORY
-  OFFLOAD_CHECK(hipHostFree(buffer->host_buffer));
-#endif
+#if defined(__OFFLOAD)
+  offloadFreeHost(buffer->host_buffer);
+  offloadFree(buffer->device_buffer);
 #else
   free(buffer->host_buffer);
 #endif
   free(buffer);
-  return;
 }
 
 /*******************************************************************************
@@ -86,7 +66,7 @@ void offload_free_buffer(offload_buffer *buffer) {
  * \author Ole Schuett
  ******************************************************************************/
 double *offload_get_buffer_host_pointer(offload_buffer *buffer) {
-  assert(buffer != NULL);
+  assert(NULL != buffer);
   return buffer->host_buffer;
 }
 
