@@ -4,11 +4,24 @@
 /*                                                                            */
 /*  SPDX-License-Identifier: BSD-3-Clause                                     */
 /*----------------------------------------------------------------------------*/
-
 #ifndef OFFLOAD_RUNTIME_H
 #define OFFLOAD_RUNTIME_H
 
-#if defined(__OFFLOAD_CUDA) || defined(__OFFLOAD_HIP)
+#if defined(__OFFLOAD_OPENCL) && !defined(__DBCSR_ACC)
+#undef __OFFLOAD_OPENCL
+#endif
+/* TODO: implement support or missing features */
+#if defined(__OFFLOAD_OPENCL)
+#if !defined(__NO_OFFLOAD_GRID)
+#define __NO_OFFLOAD_GRID
+#endif
+#if !defined(__NO_OFFLOAD_PW)
+#define __NO_OFFLOAD_PW
+#endif
+#endif
+
+#if defined(__OFFLOAD_CUDA) || defined(__OFFLOAD_HIP) ||                       \
+    defined(__OFFLOAD_OPENCL)
 #define __OFFLOAD
 
 #include <stdio.h>
@@ -19,6 +32,9 @@
 #elif defined(__OFFLOAD_HIP)
 #include <hip/hip_runtime.h>
 #include <hip/hip_version.h>
+#elif defined(__OFFLOAD_OPENCL)
+/* No relative path aka double-quote to avoid PACKAGE deps (-Iexts/dbcsr). */
+#include <src/acc/opencl/acc_opencl.h>
 #endif
 
 #ifdef __cplusplus
@@ -33,27 +49,39 @@ typedef cudaError_t offloadError_t;
 typedef hipStream_t offloadStream_t;
 typedef hipEvent_t offloadEvent_t;
 typedef hipError_t offloadError_t;
+#elif defined(__OFFLOAD_OPENCL)
+typedef void *offloadStream_t;
+typedef void *offloadEvent_t;
+typedef int offloadError_t;
 #endif
 
 #if defined(__OFFLOAD_CUDA)
 #define offloadSuccess cudaSuccess
 #elif defined(__OFFLOAD_HIP)
 #define offloadSuccess hipSuccess
+#elif defined(__OFFLOAD_OPENCL)
+#define offloadSuccess EXIT_SUCCESS
 #endif
 
 /*******************************************************************************
  * \brief Check given Cuda status and upon failure abort with a nice message.
  * \author Ole Schuett
  ******************************************************************************/
-#define OFFLOAD_CHECK(cmd)                                                     \
+#if !defined(OFFLOAD_CHECK)
+#define OFFLOAD_CHECK(CMD)                                                     \
   do {                                                                         \
-    offloadError_t error = cmd;                                                \
+    const offloadError_t error = (CMD);                                        \
     if (error != offloadSuccess) {                                             \
-      fprintf(stderr, "ERROR: %s %s %d\n", offloadGetErrorName(error),         \
-              __FILE__, __LINE__);                                             \
+      const char *const name = offloadGetErrorName(error);                     \
+      if (NULL != name && '\0' != *name) {                                     \
+        fprintf(stderr, "ERROR: \"%s\" at %s:%d\n", name, __FILE__, __LINE__); \
+      } else {                                                                 \
+        fprintf(stderr, "ERROR %i: %s:%d\n", (int)error, __FILE__, __LINE__);  \
+      }                                                                        \
       abort();                                                                 \
     }                                                                          \
   } while (0)
+#endif
 
 /*******************************************************************************
  * \brief Wrapper around cudaGetErrorName.
@@ -63,6 +91,9 @@ static inline const char *offloadGetErrorName(offloadError_t error) {
   return cudaGetErrorName(error);
 #elif defined(__OFFLOAD_HIP)
   return hipGetErrorName(error);
+#elif defined(__OFFLOAD_OPENCL)
+  (void)error; /* mark used */
+  return "";   /* TODO */
 #endif
 }
 
@@ -74,6 +105,8 @@ static inline offloadError_t offloadGetLastError(void) {
   return cudaGetLastError();
 #elif defined(__OFFLOAD_HIP)
   return hipGetLastError();
+#elif defined(__OFFLOAD_OPENCL)
+  return offloadSuccess; /* TODO */
 #endif
 }
 
@@ -87,6 +120,9 @@ static inline void offloadMemsetAsync(void *const ptr, const int val,
   OFFLOAD_CHECK(cudaMemsetAsync(ptr, val, size, stream));
 #elif defined(__OFFLOAD_HIP)
   OFFLOAD_CHECK(hipMemsetAsync(ptr, val, size, stream));
+#elif defined(__OFFLOAD_OPENCL)
+  OFFLOAD_CHECK(
+      c_dbcsr_acc_opencl_memset(ptr, val, 0 /*offset*/, size, stream));
 #endif
 }
 
@@ -98,6 +134,8 @@ static inline void offloadMemset(void *ptr, const int val, size_t size) {
   OFFLOAD_CHECK(cudaMemset(ptr, val, size));
 #elif defined(__OFFLOAD_HIP)
   OFFLOAD_CHECK(hipMemset(ptr, val, size));
+#elif defined(__OFFLOAD_OPENCL)
+  offloadMemsetAsync(ptr, val, size, NULL /*stream*/);
 #endif
 }
 
@@ -111,14 +149,15 @@ static inline void offloadMemcpyAsyncHtoD(void *const ptr1, const void *ptr2,
   OFFLOAD_CHECK(
       cudaMemcpyAsync(ptr1, ptr2, size, cudaMemcpyHostToDevice, stream));
 #elif defined(__OFFLOAD_HIP)
-
-#ifdef __OFFLOAD_UNIFIED_MEMORY
-  if (ptr1 == ptr2)
+#if defined(__OFFLOAD_UNIFIED_MEMORY)
+  if (ptr1 == ptr2) {
     return;
+  }
 #endif
-
   OFFLOAD_CHECK(
       hipMemcpyAsync(ptr1, ptr2, size, hipMemcpyHostToDevice, stream));
+#elif defined(__OFFLOAD_OPENCL)
+  OFFLOAD_CHECK(c_dbcsr_acc_memcpy_h2d(ptr2, ptr1, size, stream));
 #endif
 }
 
@@ -132,14 +171,15 @@ static inline void offloadMemcpyAsyncDtoH(void *const ptr1, const void *ptr2,
   OFFLOAD_CHECK(
       cudaMemcpyAsync(ptr1, ptr2, size, cudaMemcpyDeviceToHost, stream));
 #elif defined(__OFFLOAD_HIP)
-
-#ifdef __OFFLOAD_UNIFIED_MEMORY
-  if (ptr1 == ptr2)
+#if defined(__OFFLOAD_UNIFIED_MEMORY)
+  if (ptr1 == ptr2) {
     return;
+  }
 #endif
-
   OFFLOAD_CHECK(
       hipMemcpyAsync(ptr1, ptr2, size, hipMemcpyDeviceToHost, stream));
+#elif defined(__OFFLOAD_OPENCL)
+  OFFLOAD_CHECK(c_dbcsr_acc_memcpy_d2h(ptr2, ptr1, size, stream));
 #endif
 }
 
@@ -155,6 +195,8 @@ static inline void offloadMemcpyAsyncDtoD(void *ptr1, const void *ptr2,
 #elif defined(__OFFLOAD_HIP)
   OFFLOAD_CHECK(
       hipMemcpyAsync(ptr1, ptr2, size, hipMemcpyDeviceToDevice, stream));
+#elif defined(__OFFLOAD_OPENCL)
+  OFFLOAD_CHECK(c_dbcsr_acc_memcpy_d2d(ptr2, ptr1, size, stream));
 #endif
 }
 
@@ -166,13 +208,14 @@ static inline void offloadMemcpyHtoD(void *ptr_device, const void *ptr_host,
 #if defined(__OFFLOAD_CUDA)
   OFFLOAD_CHECK(cudaMemcpy(ptr_device, ptr_host, size, cudaMemcpyHostToDevice));
 #elif defined(__OFFLOAD_HIP)
-
-#ifdef __OFFLOAD_UNIFIED_MEMORY
-  if (ptr_device == ptr_host)
+#if defined(__OFFLOAD_UNIFIED_MEMORY)
+  if (ptr_device == ptr_host) {
     return;
+  }
 #endif
-
   OFFLOAD_CHECK(hipMemcpy(ptr_device, ptr_host, size, hipMemcpyHostToDevice));
+#elif defined(__OFFLOAD_OPENCL)
+  offloadMemcpyAsyncHtoD(ptr_device, ptr_host, size, NULL /*stream*/);
 #endif
 }
 
@@ -184,13 +227,14 @@ static inline void offloadMemcpyDtoH(void *ptr_device, const void *ptr_host,
 #if defined(__OFFLOAD_CUDA)
   OFFLOAD_CHECK(cudaMemcpy(ptr_device, ptr_host, size, cudaMemcpyDeviceToHost));
 #elif defined(__OFFLOAD_HIP)
-
-#ifdef __OFFLOAD_UNIFIED_MEMORY
-  if (ptr_device == ptr_host)
+#if defined(__OFFLOAD_UNIFIED_MEMORY)
+  if (ptr_device == ptr_host) {
     return;
+  }
 #endif
-
   OFFLOAD_CHECK(hipMemcpy(ptr_device, ptr_host, size, hipMemcpyDeviceToHost));
+#elif defined(__OFFLOAD_OPENCL)
+  offloadMemcpyAsyncDtoH(ptr_device, ptr_host, size, NULL /*stream*/);
 #endif
 }
 
@@ -205,6 +249,8 @@ static inline void offloadMemcpyToSymbol(const void *symbol, const void *src,
 #elif defined(__OFFLOAD_HIP)
   OFFLOAD_CHECK(
       hipMemcpyToSymbol(symbol, src, count, 0, hipMemcpyHostToDevice));
+#elif defined(__OFFLOAD_OPENCL)
+  assert(NULL == symbol || NULL == src || 0 == count); /* TODO */
 #endif
 }
 
@@ -216,6 +262,8 @@ static inline void offloadEventCreate(offloadEvent_t *event) {
   OFFLOAD_CHECK(cudaEventCreate(event));
 #elif defined(__OFFLOAD_HIP)
   OFFLOAD_CHECK(hipEventCreate(event));
+#elif defined(__OFFLOAD_OPENCL)
+  OFFLOAD_CHECK(c_dbcsr_acc_event_create(event));
 #endif
 }
 
@@ -227,6 +275,8 @@ static inline void offloadEventDestroy(offloadEvent_t event) {
   OFFLOAD_CHECK(cudaEventDestroy(event));
 #elif defined(__OFFLOAD_HIP)
   OFFLOAD_CHECK(hipEventDestroy(event));
+#elif defined(__OFFLOAD_OPENCL)
+  OFFLOAD_CHECK(c_dbcsr_acc_event_destroy(event));
 #endif
 }
 
@@ -238,6 +288,10 @@ static inline void offloadStreamCreate(offloadStream_t *stream) {
   OFFLOAD_CHECK(cudaStreamCreate(stream));
 #elif defined(__OFFLOAD_HIP)
   OFFLOAD_CHECK(hipStreamCreate(stream));
+#elif defined(__OFFLOAD_OPENCL)
+  int least = 0;
+  OFFLOAD_CHECK(c_dbcsr_acc_stream_priority_range(&least, NULL /*greatest*/));
+  OFFLOAD_CHECK(c_dbcsr_acc_stream_create(stream, "Offload Stream", least));
 #endif
 }
 
@@ -249,6 +303,8 @@ static inline void offloadStreamDestroy(offloadStream_t stream) {
   OFFLOAD_CHECK(cudaStreamDestroy(stream));
 #elif defined(__OFFLOAD_HIP)
   OFFLOAD_CHECK(hipStreamDestroy(stream));
+#elif defined(__OFFLOAD_OPENCL)
+  OFFLOAD_CHECK(c_dbcsr_acc_stream_destroy(stream));
 #endif
 }
 
@@ -260,6 +316,8 @@ static inline void offloadEventSynchronize(offloadEvent_t event) {
   OFFLOAD_CHECK(cudaEventSynchronize(event));
 #elif defined(__OFFLOAD_HIP)
   OFFLOAD_CHECK(hipEventSynchronize(event));
+#elif defined(__OFFLOAD_OPENCL)
+  OFFLOAD_CHECK(c_dbcsr_acc_event_synchronize(event));
 #endif
 }
 
@@ -271,6 +329,8 @@ static inline void offloadStreamSynchronize(offloadStream_t stream) {
   OFFLOAD_CHECK(cudaStreamSynchronize(stream));
 #elif defined(__OFFLOAD_HIP)
   OFFLOAD_CHECK(hipStreamSynchronize(stream));
+#elif defined(__OFFLOAD_OPENCL)
+  OFFLOAD_CHECK(c_dbcsr_acc_stream_sync(stream));
 #endif
 }
 
@@ -283,6 +343,8 @@ static inline void offloadEventRecord(offloadEvent_t event,
   OFFLOAD_CHECK(cudaEventRecord(event, stream));
 #elif defined(__OFFLOAD_HIP)
   OFFLOAD_CHECK(hipEventRecord(event, stream));
+#elif defined(__OFFLOAD_OPENCL)
+  OFFLOAD_CHECK(c_dbcsr_acc_event_record(event, stream));
 #endif
 }
 
@@ -293,7 +355,17 @@ static inline void offloadMallocHost(void **ptr, size_t size) {
 #if defined(__OFFLOAD_CUDA)
   OFFLOAD_CHECK(cudaMallocHost(ptr, size));
 #elif defined(__OFFLOAD_HIP)
+#if !defined(__OFFLOAD_UNIFIED_MEMORY)
   OFFLOAD_CHECK(hipHostMalloc(ptr, size, hipHostMallocDefault)); // inconsistent
+#else
+  assert(NULL != ptr);
+  *ptr = NULL;
+#endif
+#elif defined(__OFFLOAD_OPENCL)
+  OFFLOAD_CHECK(c_dbcsr_acc_host_mem_allocate(ptr, size, NULL /*stream*/));
+#else
+  assert(NULL != ptr);
+  *ptr = malloc(size);
 #endif
 }
 
@@ -305,6 +377,11 @@ static inline void offloadMalloc(void **ptr, size_t size) {
   OFFLOAD_CHECK(cudaMalloc(ptr, size));
 #elif defined(__OFFLOAD_HIP)
   OFFLOAD_CHECK(hipMalloc(ptr, size));
+#elif defined(__OFFLOAD_OPENCL)
+  OFFLOAD_CHECK(c_dbcsr_acc_dev_mem_allocate(ptr, size));
+#else
+  assert(NULL != ptr);
+  *ptr = NULL;
 #endif
 }
 
@@ -316,6 +393,10 @@ static inline void offloadFree(void *ptr) {
   OFFLOAD_CHECK(cudaFree(ptr));
 #elif defined(__OFFLOAD_HIP)
   OFFLOAD_CHECK(hipFree(ptr));
+#elif defined(__OFFLOAD_OPENCL)
+  OFFLOAD_CHECK(c_dbcsr_acc_dev_mem_deallocate(ptr));
+#else
+  assert(NULL == ptr);
 #endif
 }
 
@@ -326,8 +407,14 @@ static inline void offloadFreeHost(void *ptr) {
 #if defined(__OFFLOAD_CUDA)
   OFFLOAD_CHECK(cudaFreeHost(ptr));
 #elif defined(__OFFLOAD_HIP)
+#if !defined(__OFFLOAD_UNIFIED_MEMORY)
   OFFLOAD_CHECK(hipHostFree(ptr)); // inconsistent
 #endif
+#elif defined(__OFFLOAD_OPENCL)
+  OFFLOAD_CHECK(c_dbcsr_acc_host_mem_deallocate(ptr, NULL /*stream*/));
+#else
+  free(ptr);
+#endif
 }
 
 /*******************************************************************************
@@ -339,6 +426,9 @@ static inline void offloadStreamWaitEvent(offloadStream_t stream,
   OFFLOAD_CHECK(cudaStreamWaitEvent(stream, event, val));
 #elif defined(__OFFLOAD_HIP)
   OFFLOAD_CHECK(hipStreamWaitEvent(stream, event, val));
+#elif defined(__OFFLOAD_OPENCL)
+  assert(0 == val); /* TODO */
+  OFFLOAD_CHECK(c_dbcsr_acc_stream_wait_event(stream, event));
 #endif
 }
 
@@ -350,6 +440,8 @@ static inline void offloadDeviceSynchronize(void) {
   OFFLOAD_CHECK(cudaDeviceSynchronize());
 #elif defined(__OFFLOAD_HIP)
   OFFLOAD_CHECK(hipDeviceSynchronize());
+#elif defined(__OFFLOAD_OPENCL)
+  OFFLOAD_CHECK(c_dbcsr_acc_device_synchronize());
 #endif
 }
 
@@ -357,20 +449,22 @@ static inline void offloadDeviceSynchronize(void) {
  * \brief Wrapper around cudaDeviceSetLimit(cudaLimitMallocHeapSize,...).
  ******************************************************************************/
 static inline void offloadEnsureMallocHeapSize(const size_t required_size) {
-  size_t current_size;
 #if defined(__OFFLOAD_CUDA)
+  size_t current_size;
   OFFLOAD_CHECK(cudaDeviceGetLimit(&current_size, cudaLimitMallocHeapSize));
   if (current_size < required_size) {
     OFFLOAD_CHECK(cudaDeviceSetLimit(cudaLimitMallocHeapSize, required_size));
   }
 #elif defined(__OFFLOAD_HIP) && (HIP_VERSION >= 50300000)
+  size_t current_size;
   OFFLOAD_CHECK(hipDeviceGetLimit(&current_size, hipLimitMallocHeapSize));
   if (current_size < required_size) {
     OFFLOAD_CHECK(hipDeviceSetLimit(hipLimitMallocHeapSize, required_size));
   }
+#elif defined(__OFFLOAD_OPENCL)
+  assert(0 == required_size); /* TODO */
 #else
-  (void)required_size; // mark used
-  (void)current_size;
+  (void)required_size; /* mark used */
 #endif
 }
 
@@ -378,6 +472,6 @@ static inline void offloadEnsureMallocHeapSize(const size_t required_size) {
 }
 #endif
 
-#endif // defined(__OFFLOAD_CUDA) || defined(__OFFLOAD_HIP)
-
+#endif // defined(__OFFLOAD_CUDA) || defined(__OFFLOAD_HIP) ||
+       // defined(__OFFLOAD_OPENCL)
 #endif
