@@ -42,6 +42,10 @@ KEEPALIVE_SKIP_DIRS = [
 ]
 
 
+def cp2k_stem() -> str:
+    return os.getenv("CP2K_STEM", "cp2k")
+
+
 # ======================================================================================
 async def main() -> None:
     parser = argparse.ArgumentParser(description="Runs CP2K regression test suite.")
@@ -51,7 +55,8 @@ async def main() -> None:
     parser.add_argument("--num_gpus", type=int, default=0)
     parser.add_argument("--timeout", type=int, default=400)
     parser.add_argument("--maxerrors", type=int, default=50)
-    parser.add_argument("--mpiexec", default="mpiexec --bind-to none")
+    help = "Template for launching MPI jobs, {N} is replaced by number of processors."
+    parser.add_argument("--mpiexec", default="mpiexec -n {N} --bind-to none", help=help)
     help = "Runs only the first test of each directory."
     parser.add_argument("--smoketest", dest="smoketest", action="store_true", help=help)
     help = "Runs tests under Valgrind memcheck. Best used together with --keepalive."
@@ -64,6 +69,8 @@ async def main() -> None:
     parser.add_argument("--restrictdir", action="append")
     parser.add_argument("--skipdir", action="append")
     parser.add_argument("--workbasedir", type=Path)
+    parser.add_argument("--skip_unittests", action="store_true")
+    parser.add_argument("--skip_regtests", action="store_true")
     parser.add_argument("arch")
     parser.add_argument("version")
     cfg = Config(parser.parse_args())
@@ -72,7 +79,9 @@ async def main() -> None:
     start_time = time.perf_counter()
 
     # Query CP2K binary for feature flags.
-    version_bytes, _ = await (await cfg.launch_exe("cp2k", "--version")).communicate()
+    version_bytes, _ = await (
+        await cfg.launch_exe(cp2k_stem(), "--version")
+    ).communicate()
     version_output = version_bytes.decode("utf8", errors="replace")
     flags_line = re.search(r" cp2kflags:(.*)\n", version_output)
     if not flags_line:
@@ -194,17 +203,22 @@ async def main() -> None:
         print(f'PlotPoint: name="{p}th_percentile", plot="timings", ', end="")
         print(f'label="{p}th %ile", y={v:.2f}, yerr=0.0')
 
-    print("\n----------------------------- Slow Tests -------------------------------")
-    slow_test_threshold = 2 * percentile(timings, 0.95)
-    print(f"Duration threshold (2x 95th %ile): {slow_test_threshold:.2f} sec")
-    slow_tests_all = [r for r in all_results if r.duration > slow_test_threshold]
-    slow_tests = [r for r in slow_tests_all if r.fullname not in cfg.slow_suppressions]
-    num_slow_suppressed = len(slow_tests_all) - len(slow_tests)
-    print(f"Found {len(slow_tests)} slow tests ({num_slow_suppressed} suppressed):")
-    for r in slow_tests:
-        print(f"    {r.fullname :<80s} ( {r.duration:6.2f} sec)")
-    if not cfg.flag_slow:
-        slow_tests = []
+    if cfg.flag_slow:
+        print("\n" + "-" * 15 + "--------------- Slow Tests ---------------" + "-" * 15)
+        slow_test_threshold = 2 * percentile(timings, 0.95)
+        maybe_slow = [r for r in all_results if r.duration > slow_test_threshold]
+        suppressions = cfg.slow_suppressions
+        slow_reruns: List[str] = []
+        for batch in {r.batch for r in maybe_slow if r.fullname not in suppressions}:
+            print(f"Re-running {batch.name} to avoid false positives.")
+            res = (await run_batch(batch, cfg)).results
+            slow_reruns += [r.fullname for r in res if r.duration > slow_test_threshold]
+        slow_tests = [r for r in maybe_slow if r.fullname in slow_reruns]
+        num_suppressed = len([r for r in maybe_slow if r.fullname in suppressions])
+        print(f"Duration threshold (2x 95th %ile): {slow_test_threshold:.2f} sec")
+        print(f"Found {len(slow_tests)} slow tests ({num_suppressed} suppressed):")
+        for r in slow_tests:
+            print(f"    {r.fullname :<80s} ( {r.duration:6.2f} sec)")
 
     print("\n------------------------------- Summary --------------------------------")
     total_duration = time.perf_counter() - start_time
@@ -213,7 +227,7 @@ async def main() -> None:
     num_failed = sum(r.status in failure_modes for r in all_results)
     num_wrong = sum(r.status == "WRONG RESULT" for r in all_results)
     num_ok = sum(r.status == "OK" for r in all_results)
-    status_ok = (num_ok == num_tests) and (not slow_tests)
+    status_ok = (num_ok == num_tests) and (not cfg.flag_slow or not slow_tests)
     print(f"Number of FAILED  tests {num_failed}")
     print(f"Number of WRONG   tests {num_wrong}")
     print(f"Number of CORRECT tests {num_ok}")
@@ -221,7 +235,7 @@ async def main() -> None:
     summary = f"\nSummary: correct: {num_ok} / {num_tests}"
     summary += f"; wrong: {num_wrong}" if num_wrong > 0 else ""
     summary += f"; failed: {num_failed}" if num_failed > 0 else ""
-    summary += f"; slow: {len(slow_tests)}" if slow_tests else ""
+    summary += f"; slow: {len(slow_tests)}" if cfg.flag_slow and slow_tests else ""
     summary += f"; {total_duration/60.0:.0f}min"
     print(summary)
     print("Status: " + ("OK" if status_ok else "FAILED") + "\n")
@@ -241,7 +255,9 @@ class Config:
         self.num_workers = int(args.maxtasks / self.ompthreads / self.mpiranks)
         self.workers = Semaphore(self.num_workers)
         self.cp2k_root = Path(__file__).resolve().parent.parent
-        self.mpiexec = args.mpiexec.split()
+        self.mpiexec = args.mpiexec
+        if "{N}" not in self.mpiexec:  # backwards compatibility
+            self.mpiexec = f"{self.mpiexec} ".replace(" ", " -n {N} ", 1).strip()
         self.smoketest = args.smoketest
         self.valgrind = args.valgrind
         self.keepalive = args.keepalive
@@ -252,6 +268,8 @@ class Config:
         self.max_errors = args.maxerrors
         self.restrictdirs = args.restrictdir if args.restrictdir else [".*"]
         self.skipdirs = args.skipdir if args.skipdir else []
+        self.skip_unittests = args.skip_unittests
+        self.skip_regtests = args.skip_regtests
         datestamp = datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
         leaf_dir = f"TEST-{args.arch}-{args.version}-{datestamp}"
         self.work_base_dir = (
@@ -294,11 +312,17 @@ class Config:
             env["CUDA_VISIBLE_DEVICES"] = ",".join(visible_gpu_devices)
             env["HIP_VISIBLE_DEVICES"] = ",".join(visible_gpu_devices)
         env["OMP_NUM_THREADS"] = str(self.ompthreads)
-        cmd = [str(self.cp2k_root / "exe" / self.arch / f"{exe_stem}.{self.version}")]
+        exe_path = Path(f"{exe_stem}.{self.version}")
+        if exe_path.is_absolute():
+            cmd = [str(exe_path)]
+        else:
+            cmd = [
+                str(self.cp2k_root / "exe" / self.arch / f"{exe_stem}.{self.version}")
+            ]
         if self.valgrind:
             cmd = ["valgrind", "--error-exitcode=42", "--exit-on-first-error=yes"] + cmd
         if self.use_mpi:
-            cmd = [self.mpiexec[0], "-n", str(self.mpiranks)] + self.mpiexec[1:] + cmd
+            cmd = self.mpiexec.format(N=self.mpiranks).split() + cmd
         if self.debug:
             print(f"Creating subprocess: {cmd} {args}")
         return asyncio.create_subprocess_exec(
@@ -430,7 +454,9 @@ class Cp2kShell:
 
     async def start(self) -> None:
         assert self._child is None
-        self._child = await self.cfg.launch_exe("cp2k", "--shell", cwd=self.workdir)
+        self._child = await self.cfg.launch_exe(
+            cp2k_stem(), "--shell", cwd=self.workdir
+        )
         await self.ready()
         await self.sendline("HARSH")  # With harsh mode any error leads to an abort.
         await self.ready()
@@ -496,7 +522,11 @@ async def wait_for_child_process(
 # ======================================================================================
 async def run_batch(batch: Batch, cfg: Config) -> BatchResult:
     async with cfg.workers:
-        results = (await run_unittests(batch, cfg)) + (await run_regtests(batch, cfg))
+        results = []
+        if not cfg.skip_unittests:
+            results += await run_unittests(batch, cfg)
+        if not cfg.skip_regtests:
+            results += await run_regtests(batch, cfg)
         return BatchResult(batch, results)
 
 
@@ -571,7 +601,7 @@ async def run_regtests_classic(batch: Batch, cfg: Config) -> List[TestResult]:
     for test in batch.regtests:
         start_time = time.perf_counter()
         start_dirsize = dirsize(batch.workdir)
-        child = await cfg.launch_exe("cp2k", test.inp_fn, cwd=batch.workdir)
+        child = await cfg.launch_exe(cp2k_stem(), test.inp_fn, cwd=batch.workdir)
         output, returncode, timed_out = await wait_for_child_process(child, cfg.timeout)
         test.out_path.write_bytes(output)
         duration = time.perf_counter() - start_time
@@ -601,33 +631,49 @@ def eval_regtest(
     output = output_bytes.decode("utf8", errors="replace")
     output_tail = "\n".join(output.split("\n")[-100:])
     error = "x" * 100 + f"\n{test.out_path}\n"
+
+    # check for timeout
     if timed_out:
         error += f"{output_tail}\n\nTimed out after {duration} seconds."
         return TestResult(batch, test, duration, "TIMED OUT", error=error)
-    elif returncode != 0:
+
+    # check for crash
+    if returncode != 0:
         error += f"{output_tail}\n\nRuntime failure with code {returncode}."
         return TestResult(batch, test, duration, "RUNTIME FAIL", error=error)
-    elif output_size > 2 * 1024 * 1024 and not is_huge_suppressed:  # 2MiB limit
+
+    # check for huge output
+    if output_size > 2 * 1024 * 1024 and not is_huge_suppressed:  # 2MiB limit
         error += f"Test produced {output_size/1024/1024:.2f} MiB of output."
         return TestResult(batch, test, duration, "HUGE OUTPUT", error=error)
-    elif not test.test_type:
-        return TestResult(batch, test, duration, "OK")  # test type zero
-    else:
-        value_txt = test.test_type.grep(output)
-        if not value_txt:
-            error += f"{output_tail}\n\nResult not found: '{test.test_type.pattern}'."
-            return TestResult(batch, test, duration, "WRONG RESULT", error=error)
-        else:
-            # compare result to reference
-            value = float(value_txt)
-            diff = value - test.ref_value
-            rel_error = abs(diff / test.ref_value if test.ref_value != 0.0 else diff)
-            if rel_error <= test.tolerance:
-                return TestResult(batch, test, duration, "OK", value)
-            else:
-                error += f"Difference too large: {rel_error:.2e} > {test.tolerance}, "
-                error += f"ref_value: {test.ref_value_txt}, value: {value_txt}."
-                return TestResult(batch, test, duration, "WRONG RESULT", value, error)
+
+    # happy end for test type zero
+    if not test.test_type:
+        return TestResult(batch, test, duration, "OK")
+
+    # grep result
+    value_txt = test.test_type.grep(output)
+    if not value_txt:
+        error += f"{output_tail}\n\nResult not found: '{test.test_type.pattern}'."
+        return TestResult(batch, test, duration, "WRONG RESULT", error=error)
+
+    # parse result
+    try:
+        value = float(value_txt)
+    except:
+        error += f"Could not parse result as float: '{value_txt}'."
+        return TestResult(batch, test, duration, "WRONG RESULT", error=error)
+
+    # compare result to reference
+    diff = value - test.ref_value
+    rel_error = abs(diff / test.ref_value if test.ref_value != 0.0 else diff)
+    if rel_error > test.tolerance:
+        error += f"Difference too large: {rel_error:.2e} > {test.tolerance}, "
+        error += f"ref_value: {test.ref_value_txt}, value: {value_txt}."
+        return TestResult(batch, test, duration, "WRONG RESULT", value, error)
+
+    # happy end
+    return TestResult(batch, test, duration, "OK", value)
 
 
 # ======================================================================================
