@@ -86,12 +86,16 @@ def main() -> None:
                 continue
             if root.startswith("./tools/prettify/fprettify"):
                 continue
+            if root.startswith("./tools/precommit/fprettify"):
+                continue
             if root.startswith("./tools/build_utils/fypp"):
                 continue
             if root.startswith("./tools/autotools"):
                 continue
             if root.startswith("./tools/minimax_tools/1_xData"):
                 continue
+            if root.startswith("./tools/fedora"):
+                continue
             if root.startswith("./data/DFTB/scc"):
                 continue
             if root.startswith("./arch"):
@@ -192,9 +196,9 @@ def process_file(fn: str, allow_modifications: bool) -> None:
 
     if re.match(r".*\.(F|fypp)$", fn):
         run_local_tool("./tools/doxify/doxify.sh", fn)
-        run_prettify(fn)
+        run_format_fortran(fn)
 
-    if re.match(r".*\.(c|cu|h)$", fn):
+    if re.match(r".*\.(c|cu|cl|h)$", fn):
         run_remote_tool("clangformat", fn)
 
     if re.match(r".*\.(cc|cpp|cxx|hcc|hpp|hxx)$", fn):
@@ -246,13 +250,13 @@ def process_file(fn: str, allow_modifications: bool) -> None:
 
 
 # ======================================================================================
-def run_prettify(fn: str) -> None:
+def run_format_fortran(fn: str) -> None:
     if fn in ("./src/base/base_uses.f90", "./src/common/util.F"):
         return  # Skipping because of prettify bugs.
 
     # The prettify tool processes only about 1k lines of code per second.
     # Hence, setting a generous timeout as our largest file has 100k lines.
-    run_local_tool("./tools/prettify/prettify.py", fn, timeout=600)
+    run_local_tool("./tools/precommit/format_fortran.py", fn, timeout=600)
 
 
 # ======================================================================================
